#ifndef GUARD_fileentry_h
#define GUARD_fileentry_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file fileentry.h
/// @brief Class for filename entry and browse button header file.

#include <gtkmm/box.h>
#include <gtkmm/entry.h>
#include <gtkmm/button.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/filechooserdialog.h>

#include <libintl.h>

/// @brief Class for filename entry and browse button.
/// @todo Replace it to FileChooserButton when change to gtkmm 2.5.x over(coressponding Gtk+ 2.6 over).
class FileEntry : public Gtk::HBox
{
public:
    FileEntry(const char* tooltipText, const char* dialogTitle)
        : browseButton_(gettext("Browse...")),
          dlgTitle_(dialogTitle),
          borderWidth_(12)
    {
        entryTooltips_.set_tip(filenameEntry_, tooltipText);

        browseButton_.signal_clicked().connect(sigc::mem_fun(*this, &FileEntry::ShowFileChooserDialog) );

        pack_start(filenameEntry_, Gtk::PACK_SHRINK);
        pack_start(browseButton_, Gtk::PACK_SHRINK);
    }

    ~FileEntry() {}

    void SetEntryText(const Glib::ustring& text)
    {
        filenameEntry_.set_text(text);
    }

    Glib::ustring GetEntryText() const
    {
        return filenameEntry_.get_text();
    }

private:
    Gtk::Entry filenameEntry_;
    Gtk::Button browseButton_;
    Gtk::Tooltips entryTooltips_;

    Glib::ustring dlgTitle_;

    const unsigned int borderWidth_;

    void ShowFileChooserDialog();

    virtual void on_size_request(Gtk::Requisition* requisition);

    virtual void on_size_allocate(Gtk::Allocation& allocation);

    void ResetRequisition(Gtk::Requisition* req)
    {
        *req = Gtk::Requisition();
    }

    void CalcWidgetSize(Gtk::Requisition* orig);

    void RemoveLastBorder(Gtk::Requisition* req)
    {
        req->width -= borderWidth_;
    }

    void SetHeight(Gtk::Requisition* orig, const Gtk::Requisition& dest)
    {
        if(dest.height > orig->height)
            orig->height = dest.height;
    }

    void SetWidth(Gtk::Requisition* orig, const Gtk::Requisition& dest)
    {
        orig->width += dest.width + borderWidth_;
    }

    Gtk::Requisition GetWidgetSizeRequest(Gtk::Widget* widget)
    {
        Gtk::Requisition req;

        widget->size_request(req);

        return req;
    }

    Gtk::Allocation CreateAllocation(int x, int y, int width, int height)
    {
        Gtk::Allocation rtn;

        rtn.set_x(x);
        rtn.set_y(y);
        rtn.set_width(width);
        rtn.set_height(height);

        return rtn;
    }

    void SetSizeAllocate(Gtk::Widget* widget, int x, int y, int height)
    {
        widget->size_allocate(CreateAllocation(x, y, GetWidgetSizeRequest(widget).width, height));
    }

    void IncreaseXpos(int& x, Gtk::Widget* widget)
    {
        x += GetWidgetSizeRequest(widget).width;
        x += borderWidth_;
    }
};

#endif
