#ifndef GUARD_mixer_h
#define GUARD_mixer_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file mixer.h
/// @brief Class for OSS mixer header file

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_SYS_SOUNDCARD_H
#include <sys/soundcard.h>
#else
#ifdef HAVE_MACHINE_SOUNDCARD_H
#include <machine/soundcard.h>
#else
#ifdef HAVE_SOUNDCARD_H
#include <soundcard.h>
#endif
#endif
#endif

#include <string>

namespace Audio {

    class Mixer
    {
    public:
        virtual ~Mixer() {}

        virtual double GetVolumn() = 0;

        virtual void SetVolumn(double) = 0;

        virtual void SetDevice(const std::string& path) = 0;
    };

    double Increase(double orig, double step = 5.0);
    double Decrease(double orig, double step = 5.0);

    void IncreaseVolumn(Mixer& mixer);
    void DecreaseVolumn(Mixer& mixer);

    /// @brief Class for OSS mixer
    class OSSMixer : public Mixer
    {
    public:
        OSSMixer(const std::string& dev)
            : devicePath_(dev), currentPCMVolumn_(0.0)
        {}

        double GetVolumn();

        void SetVolumn(double);

        void SetDevice(const std::string& path);

    private:
        OSSMixer(const OSSMixer&);
        OSSMixer& operator=(const OSSMixer&);

        std::string devicePath_;
        double currentPCMVolumn_;
    };
}

inline double Audio::Increase(double orig, double step)
{
    return orig > (100.0 - step) ? 100.0 : orig + step;
}

inline double Audio::Decrease(double orig, double step)
{
    return orig < step ? 0.0 : orig - step;
}

inline void Audio::IncreaseVolumn(Mixer& mixer)
{
    mixer.SetVolumn(Increase(mixer.GetVolumn()));
}

inline void Audio::DecreaseVolumn(Mixer& mixer)
{
    mixer.SetVolumn(Decrease(mixer.GetVolumn()));
}

#endif
