#ifndef GUARD_player_aac_h
#define GUARD_player_aac_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file player_aac.h
/// @brief Class for mpeg 4 audio decoding using FAAD2 header file.
/// @todo Deal mp4ff.h more smartly.

#include <cstdio>

#include <glibmm.h>

#include "aacbuffer.h"

#include "misc.h"

#include "player.h"

#include "fwd.h"

namespace FAAD {

    /// @brief Class for mpeg 4 audio decoding using FAAD2.
    /// @todo Code clean up and real testing.
    template <class Buffer>
    class AACPlayer : public Player
    {
    public:
        AACPlayer(Buffer& buffer, const std::string& filename);
        ~AACPlayer();
        
    private:
        AACPlayer(const AACPlayer& aacPlayer);
        AACPlayer& operator=(const AACPlayer& aacPlayer);

        Glib::Dispatcher signalFinish_;

        DecodeState decodeState_;

        unsigned long seekSeconds_;

        const unsigned int maxChannels_;
        unsigned int frameSize_;
	    
        FileHandler inFile_;
        bool mp4File_;

        void* sampleBuffer_;

        Buffer& buffer_;
        PlayerInterface& playerInterface_;

        void DoStart();
        void DoStop();
        void DoSeek(unsigned long time);

        void MP4Decode();
        void AACDecode();

        void SetDecoderConfig(faacDecHandle& decoder, const unsigned char& defObjectType = LC, const unsigned char& outputFormat = FAAD_FMT_DOUBLE, const unsigned char& downMatrix = 1, const unsigned char& useOldADTSFormat = 0, const unsigned char& dontUpSampleImplicitSBR = 1);
	    
    };
}

template <class Buffer>
std::auto_ptr<Player> CreateAACPlayer(Buffer& buffer, const std::string& filename);

#endif
