#ifndef GUARD_player_mp3http_h
#define GUARD_player_mp3http_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file player_mp3http.h
/// @brief Class for play mp3 on INET header file.

#include <unistd.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>

#include <string>

#include <glibmm.h>

#include <mad.h>

#include <ace/INET_Addr.h>
#include <ace/SOCK_Stream.h>
#include <ace/SOCK_Connector.h>

#include "player.h"
#include "player_mp3.h"
#include "connector.h"

#include "fwd.h"

namespace MAD {

    struct StreamBuffer {

        unsigned char* data;
        unsigned long length;

    };

    // gcc 3.3.x cannot compile these variable as template static const
    // variable :( gcc 3.4.x works fine, so change it later or not.
    static const unsigned int recvBufMaxSize = 131072;
    static const unsigned int recvBufLimitSize = 98304;
    static const unsigned int inputBufSize = 32768;

    /// @brief Class for play mp3 on HTTP.
    ///
    /// MAD requires static function, almost member functions and variables are static.
    template <class Buffer>
    class MP3HttpPlayer : public Player 
    {
    public:
        MP3HttpPlayer(Buffer& buffer, const std::string& url);
        ~MP3HttpPlayer()
        {}

    private:
        MP3HttpPlayer& operator=(const MP3HttpPlayer& mp3HttpPlayer);
        MP3HttpPlayer(const MP3HttpPlayer& mp3HttpPlayer);

        static Buffer* buffer_;	
        static PlayerInterface* playerInterface_;
        static MainManager* mainManager_;		///< Program control MainManager
	    
        Glib::Thread* recvLoopThread_;
        static Glib::Mutex* recvLoopMutex_;
        static Glib::Cond* recvLoopCondPush_;
        static Glib::Cond* recvLoopCondPop_;

        Glib::Dispatcher signalFinish_;		///< Dispatcher for finish signal
        static Glib::Dispatcher signalUpdateStatusbar_;
        static Glib::Dispatcher signalUpdateStatusbarTimeoutClear_;
        static Glib::Dispatcher signalClearStatusbar_;

        struct mad_decoder decoder_;
        struct StreamBuffer buf_;
	    
        static DecodeState decodeState_;
        static unsigned int recvBufSize_;

        static bool inRecving_;
        static bool disconnected_;

        static char recvBuf_[recvBufMaxSize];
        static char inputBuf_[inputBufSize];

        static ACE_INET_Addr destAddress_;
        static ACE_SOCK_Connector connector_;
        static ACE_SOCK_Stream peer_;

        std::string destUrl_;
        
        void DoStart();

        void DoStop(); 
	    
        void DoSeek(unsigned long time);

        void Init();

        void Clean();
	    
        void RecvLoop();

        NetUtil::HTTPHeaderStatus GetHeader();

        void AllocateThreadVariables()
        {
            recvLoopMutex_ = new Glib::Mutex;
            recvLoopCondPush_ = new Glib::Cond;
            recvLoopCondPop_ = new Glib::Cond;
        }

        void DeallocateThreadVariables()
        {
            delete recvLoopCondPop_;
            recvLoopCondPop_ = 0;

            delete recvLoopCondPush_;
            recvLoopCondPush_ = 0;

            delete recvLoopMutex_;
            recvLoopMutex_ = 0;
        }

        static enum mad_flow Input(void* data, struct mad_stream* stream);
        static enum mad_flow Output(void* data, struct mad_header const* header, struct mad_pcm* pcm);
        static enum mad_flow Header(void* data, struct mad_header const* header);
    };

}

template <class Buffer>
std::auto_ptr<Player> CreateMP3HttpPlayer(Buffer& buffer, const std::string& url);

#endif
