#ifndef GUARD_player_wma_h
#define GUARD_player_wma_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// wma decoder came from bmp-wma plugin.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file player_wma.h
/// @brief Class for play wma header file.

#include <string>

#include <glibmm.h>

#include "player.h"

#include "fwd.h"

#include "../src/libffwma/avcodec.h"
#include "../src/libffwma/avformat.h"

/// @brief Namespace for class that use FFMPEG(libffwma) library.
namespace FFMPEG {

    /// @brief Class for play wma.
    template <class Buffer>
    class WMAPlayer : public Player
    {
    public:
        WMAPlayer(Buffer& buffer, const std::string& filename);
        ~WMAPlayer()
        {}

    private:
        WMAPlayer& operator=(const WMAPlayer& wmaPlayer);
        WMAPlayer(const WMAPlayer& wmaPlayer);

        Glib::Dispatcher signalFinish_;	    ///< Dispatcher for finish signal.

        Glib::Mutex avMutex_;		    ///< Mutex to protect decodeState.

        DecodeState decodeState_;		    ///< DecodeState to check status.

        unsigned long int seekSeconds_;	    ///< Wanted seek time in seconds.
        const unsigned int bufSize_;			    ///< Buffer size.
        const unsigned int count_;
        int index_;	

        Buffer& buffer_;
        PlayerInterface& playerInterface_;

        AVCodec* avCodec_;
        AVPacket avPacket_;
        AVCodecContext* avCodecContext_;
        AVFormatContext* avFormatContext_;
	    
        void DoStart();
        void DoStop();
        void DoSeek(unsigned long time);

        void AVCodecSelect();
        void Clean()
        {
            avcodec_close(avCodecContext_);
            av_close_input_file(avFormatContext_);
        }
    };
}

template <class Buffer>
std::auto_ptr<Player> CreateWMAPlayer(Buffer& buffer, const std::string& filename);

#endif
