#ifndef GUARD_tag_aac_h
#define GUARD_tag_aac_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file tag_aac.h
/// @brief Class for mpeg 4 audio tag processing header file.

#include <cstdio>
#include <string>
#include <map>

#include <exception>

#include "tag.h"
#include "aacbuffer.h"
#include "misc.h"

/// @brief Class for mpeg 4 audio tag processing header file.
class AACTag : public Tag
{
public:
    AACTag(const std::string& filename);
    ~AACTag()
    {}

private:
    void ProcessAAC(FILE* file);
    void ProcessMP4(FILE* file);

    virtual void ProcessYear();
    virtual void ProcessTrack();
    virtual void ProcessAlbum();
    virtual void ProcessGenre();
    virtual void ProcessArtist();
    virtual void ProcessComment();
};

std::auto_ptr<Tag> CreateAACTag(const std::string& filename);

/// @brief Class for MP4 type file.
class MP4
{
public:
    MP4(FILE* file);
    ~MP4()
    {
        mp4ff_close(mp4ffFile_);

        faacDecClose(decoder_);
    }
    
    void GetTotalTimeSeconds(unsigned long& totalTimeSeconds);
		
    void GetTitle(Glib::ustring& title)
    {
        GetMetaData("title", title);
    }

    void GetArtist(Glib::ustring& artist)
    {
        GetMetaData("artist", artist);
    }

    void GetAlbum(Glib::ustring& album)
    {
        GetMetaData("album", album);
    }

    void GetTrack(Glib::ustring& track)
    {
        GetMetaData("track", track);
    }

    void GetDate(Glib::ustring& date)
    {
        GetMetaData("date", date);
    }

    void GetGenre(Glib::ustring& genre)
    {
        GetMetaData("genre", genre);
    }

    void GetComment(Glib::ustring& comment)
    {
        GetMetaData("comment", comment);
    }

private:
    MP4(const MP4& mp4);
    MP4& operator=(const MP4& mp4);

    int track_;
    unsigned int frameSize_;

    faacDecHandle decoder_;
    mp4ff_t* mp4ffFile_;
    mp4AudioSpecificConfig mp4ASC_;

    const std::auto_ptr<mp4ff_callback_t> mp4Cb_;
        
    typedef int (*Mp4ffMetaFunction)(const mp4ff_t*, char**);
    typedef std::map<std::string, Mp4ffMetaFunction> Mp4ffMetaCallbacks;
    Mp4ffMetaCallbacks mp4ffMetaCallbacks_;

    void Init();
    void GetMetaData(const char* const key, Glib::ustring& output);
};

#endif
