#ifndef GUARD_trayicon_h
#define GUARD_trayicon_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file trayicon.h
/// @brief Class for trayicon header file

#include <glibmm.h>
#include <gtkmm/eventbox.h>
#include <gtkmm/image.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/menuitem.h>

#include <sigc++/sigc++.h>

extern "C"
{

#include "../include/eggtrayicon.h"

}

#include "gtkmmutil.h"

class Command;

/// @brief Class for trayicon
class TrayIcon
{
public:
    TrayIcon(const char* imageFile, const char* id);
    virtual ~TrayIcon();

    /// @brief Set tooltips for display status.
    void SetTooltips(const Glib::ustring& string)
    {
        trayIconToolTips_.set_tip(trayIconEventBox_, string);
    }

protected:
    RealizableMenu trayIconMenu_;

    sigc::signal0<void> signalFirstBtnPress_;
    sigc::signal0<void> signalWheelUp_;
    sigc::signal0<void> signalWheelDown_;

private:
    Gtk::EventBox trayIconEventBox_;
    Gtk::Image trayIconImage_;
    Gtk::Tooltips trayIconToolTips_;

    EggTrayIcon* eggTrayIcon_;

    void EventBoxInit();

    bool OnScroll(GdkEventScroll* ev);
    bool OnBtnPress(GdkEventButton* ev);

    void GetWindowOrigin(int& x, int& y);
    int GetWindowHeight();

    void GetMenuSize(int& width, int& height);
    int GetMenuWidth();
    int GetMenuHeight();

    int GetTrayIconWidth();
    int GetTrayIconHeight();

    void MenuPositionCalc(int& x, int& y, bool& pushIn);
};

class SaguaTrayIcon : public TrayIcon
{
public:
    SaguaTrayIcon(Command& command);
    ~SaguaTrayIcon()
    {}

private:
    SaguaTrayIcon& operator=(const SaguaTrayIcon&);
    SaguaTrayIcon(const SaguaTrayIcon&);

    Gtk::MenuItem mainWndOpenMenu_;

    void ConnectSignals(Command& command);
    void InitMenu(Command& command);
};

#endif
