/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file itemselector.cc
/// @brief Class to get item from playlist source file.

#include "../include/itemselector.h"

ItemSelector::~ItemSelector()
{}

bool RepeatSelector::GetNextInternal(std::string& item)
{
    return GetCircularNext(item);
}

bool RepeatSelector::GetPrevInternal(std::string& item)
{
    return GetCircularPrev(item);
}

bool RepeatSelector::GetNext(std::string& item)
{
    return GetCircularNext(item);
}

bool RepeatSelector::GetPrev(std::string& item)
{
    return GetCircularPrev(item);
}

bool NormalSelector::GetNextInternal(std::string& item)
{
    return GetOnewayNext(item);
}

bool NormalSelector::GetPrevInternal(std::string& item)
{
    return GetOnewayPrev(item);
}

bool NormalSelector::GetNext(std::string& item)
{
    item = getItem_.GetNext(listState_);

    if(Sagua::LIST_EMPTY == listState_)
        return false;

    if(Sagua::LIST_LAST == listState_)
        item = getItem_.GetLast(listState_);

    return true;
}

bool NormalSelector::GetPrev(std::string& item)
{
    item = getItem_.GetPrev(listState_);

    if(Sagua::LIST_EMPTY == listState_)
        return false;

    if(Sagua::LIST_FIRST == listState_)
        item = getItem_.GetFirst(listState_);

    return true;
}

bool RandomSelector::GetNextInternal(std::string& item)
{
    return GetRandom(item);
}

bool RandomSelector::GetPrevInternal(std::string& item)
{
    return GetRandom(item);
}

bool RandomSelector::GetNext(std::string& item)
{
    return GetRandom(item);
}

bool RandomSelector::GetPrev(std::string& item)
{
    return GetRandom(item);
}

bool EmptySelector::GetNextInternal(std::string& item)
{
    return false;
}

bool EmptySelector::GetPrevInternal(std::string& item)
{
    return false;
}

bool EmptySelector::GetNext(std::string& item)
{
    return false;
}

bool EmptySelector::GetPrev(std::string& item)
{
    return false;
}

std::auto_ptr<ItemSelector> ItemSelectorCreator::Create(unsigned int type, GetItem& getItem)
{
    switch(type) {
    case NORMAL :
        return std::auto_ptr<ItemSelector>(new NormalSelector(getItem));

    case REPEAT :
        return std::auto_ptr<ItemSelector>(new RepeatSelector(getItem));

    case RANDOM :
        return std::auto_ptr<ItemSelector>(new RandomSelector(getItem));

    case EMPTY :
    default :
        return std::auto_ptr<ItemSelector>(new EmptySelector(getItem));
    }
}
