/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file laststate.cc
/// @brief Class for load/save last window state source file.

#include <cstdio>
#include <map>

#include "../include/laststate.h"
#include "../include/misc.h"

LastState::LastState()
    : windowWidth_(300),
      windowHeight_(330),
      playRepeat_(false),
      playRepeatCurrent_(false),
      playRandom_(false),
      viewBrowser_(false),
      viewList_(true),
      viewCompact_(false),
      viewStatusBar_(true),
      lastBrowseDirectory_(ProgramFile::Instance().GetDir())
{
    Load();
}

void LastState::Load()
{
    std::map<std::string, std::string> configMap;
    // for directory entry, load configfile not lowercase!
    if(ConfigfileToMap(ProgramFile::Instance().GetLastState(), configMap, false, false)) {

        ReadConfigMap(configMap, "PlayRepeat", playRepeat_);
        ReadConfigMap(configMap, "PlayRepeatCurrent", playRepeatCurrent_);
        ReadConfigMap(configMap, "PlayRandom", playRandom_);
        ReadConfigMap(configMap, "LastBrowseDirectory", lastBrowseDirectory_);
        ReadConfigMap(configMap, "WindowWidth", windowWidth_);
        ReadConfigMap(configMap, "WindowHeight", windowHeight_);
        ReadConfigMap(configMap, "ViewStatusBar", viewStatusBar_);
        ReadConfigMap(configMap, "ViewBrowser", viewBrowser_);
        ReadConfigMap(configMap, "ViewList", viewList_);
        ReadConfigMap(configMap, "ViewCompact", viewCompact_);
    }
}

void LastState::Save()
{
    if(!ProgramFile::Instance().CheckDir())
        return;
	
    FILE* lastStateFile = std::fopen(ProgramFile::Instance().GetLastState(), "w");

    if(lastStateFile) {

        std::fprintf(lastStateFile, "WindowWidth = %d\n", windowWidth_);
        std::fprintf(lastStateFile, "WindowHeight = %d\n", windowHeight_);
        std::fprintf(lastStateFile, "PlayRepeat = %d\n", playRepeat_);
        std::fprintf(lastStateFile, "PlayRepeatCurrent = %d\n", playRepeatCurrent_);
        std::fprintf(lastStateFile, "PlayRandom = %d\n", playRandom_);
        std::fprintf(lastStateFile, "ViewBrowser = %d\n", viewBrowser_);
        std::fprintf(lastStateFile, "ViewList = %d\n", viewList_);
        std::fprintf(lastStateFile, "ViewCompact = %d\n", viewCompact_);
        std::fprintf(lastStateFile, "ViewStatusBar = %d\n", viewStatusBar_);
        std::fprintf(lastStateFile, "LastBrowseDirectory = %s\n", lastBrowseDirectory_.c_str());

        std::fclose(lastStateFile);
    }
}
