/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file prefwin.cc
/// @brief Class for user preference window source file.

#include <iostream>
#include <fstream>

#include <map>
#include <string>

#include <libintl.h>

#include <gtkmm/main.h>
#include <gtkmm/window.h>
#include <gtkmm/image.h>
#include <gtkmm/box.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/notebook.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/table.h>
#include <gtkmm/entry.h>
#include <gtkmm/radiobuttongroup.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/scale.h>
#include <gtkmm/label.h>
#include <gtkmm/stock.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/menu.h>
#include <gtkmm/optionmenu.h>
#include <gtkmm/combo.h>

#include "../include/fileentry.h"

#include "../include/prefwin.h"
#include "../include/mainmanager.h"
#include "../include/preference.h"
#include "../include/misc.h"
#include "../include/trace.h"

/// @brief General tab of Preference window.
class PrefWndGeneralTab
{
public:
    PrefWndGeneralTab(SaguaPreference& pref);
    ~PrefWndGeneralTab()
    {}

    void AppendTo(Gtk::Notebook& notebook, const char* label)
    {
        notebook.append_page(generalTable_, label);
    }

    bool CompactBgPixmapChanged() const
    {
        return compactBgChanged_;
    }

    bool MixerDevChanged() const
    {
        return mixerChanged_;
    }

    bool AssumeCharsetChanged() const
    {
        return assumeCharsetChanged_;
    }

    void Update();

private:
    PrefWndGeneralTab(const PrefWndGeneralTab& prefWndGeneralTab);
    PrefWndGeneralTab& operator=(const PrefWndGeneralTab& prefWndGeneralTab);

    Gtk::Table generalTable_;

    FileEntry compactBgPixmap_;

    Gtk::Menu trayIconActionMenu_;
    Gtk::OptionMenu trayIconActionOptionMenu_;

    Gtk::Entry mixerDev_;
    Gtk::Entry assumeCharset_;

    Gtk::Label titleLabel_;
    Gtk::Label trayIconActionLabel_;
    Gtk::Label assumeCharsetLabel_;
    Gtk::Label compactBgPixmapLabel_;
    Gtk::Label mixerDevLabel_;

#ifdef ENABLE_HTTP
    Gtk::CheckButton convertUrlBtn_;
#endif

    Gtk::Tooltips trayIconActionTooltips_;
    Gtk::Tooltips assumeCharsetTooltips_;

    SaguaPreference& pref_;

    bool mixerChanged_;
    bool compactBgChanged_;
    bool assumeCharsetChanged_;

    void OnMenuTrayIconAction(const char* action);
};

/// @brief Audio driver setting tab of Preference window.
class PrefWndAudioDriverTab
{
public:
    PrefWndAudioDriverTab(MainManager& mainManager);
    ~PrefWndAudioDriverTab()
    {}

    void AppendTo(Gtk::Notebook& notebook, const char* label)
    {
        notebook.append_page(audioDriverWindow_, label);
    }

    bool AudioDriverChanged() const
    {
        return audioChanged_;
    }

    bool CheckDriverChange();

    void Update();

private:
    PrefWndAudioDriverTab(const PrefWndAudioDriverTab& prefWndAudioDriverTab);
    PrefWndAudioDriverTab& operator=(const PrefWndAudioDriverTab& prefWndAudioDriverTab);

    Gtk::ScrolledWindow audioDriverWindow_;

    Gtk::Image warningIcon_;

    Gtk::HBox alsa5CardOptionBox_, alsa5DevOptionBox_;
    Gtk::HBox alsa9OptionBox_; 
    Gtk::HBox ossOptionBox_;
    Gtk::HBox sunOptionBox_, aixsOptionBox_;
    Gtk::HBox warningBox_;

    Gtk::Table mainTable_;

    Gtk::Entry alsa5Card_, alsa5Dev_;
    Gtk::Entry alsa9Dev_;
    Gtk::Entry ossDsp_;
    Gtk::Entry sunDev_;
    Gtk::Entry aixsDev_;

    Gtk::RadioButtonGroup driverBtnGroup_;
    Gtk::RadioButton defaultBtn_;
    Gtk::RadioButton alsa5Btn_;
    Gtk::RadioButton alsa9Btn_;
    Gtk::RadioButton esdBtn_;
    Gtk::RadioButton irixBtn_;
    Gtk::RadioButton ossBtn_;
    Gtk::RadioButton sunBtn_;
    Gtk::RadioButton aixsBtn_;

    Gtk::Button testBtn_;

    Gtk::Label titleLabel_;
    Gtk::Label alsa5CardLabel_;
    Gtk::Label alsa5DevLabel_;
    Gtk::Label alsa9DevLabel_;
    Gtk::Label ossDspLabel_;
    Gtk::Label sunDevLabel_;
    Gtk::Label aixsDevLabel_;
    Gtk::Label testLabel_;
    Gtk::Label testResultLabel_;
    Gtk::Label warningLabel_;

    Gtk::Tooltips alsa5Tooltips_;
    Gtk::Tooltips alsa9Tooltips_;
    Gtk::Tooltips alsa9DeviceTooltips_;

    std::string selectedDriver_;

    MainManager& mainManager_;

    bool audioChanged_;

    void OnBtnAudioDriver(const char* driver);
    void OnBtnAudioDriverTest();
};

/// @brief Effector tab of Preference window.
class PrefWndEffectorTab
{
public:
    PrefWndEffectorTab(SaguaPreference& pref);
    ~PrefWndEffectorTab()
    {}

    void AppendTo(Gtk::Notebook& notebook, const char* label)
    {
        notebook.append_page(effectorTable_, label);
    }

    bool EffectChanged() const
    {
        return effectChanged_;
    }

    bool CheckEffectChange();

    void Update();

private:
    PrefWndEffectorTab(const PrefWndEffectorTab& prefWndEffectorTab);
    PrefWndEffectorTab& operator=(const PrefWndEffectorTab& prefWndEffectorTab);

    Gtk::Table effectorTable_;

    Gtk::HBox stereoenhanceBox_;
    Gtk::HBox roomBox_;
    Gtk::HBox room2Box_;
    Gtk::HBox harmonyBox_;
    Gtk::HBox rhythmeBox_;
    Gtk::HBox distortionBox_;

    Gtk::CheckButton bassboostBtn_;
    Gtk::CheckButton bassboost2Btn_;
    Gtk::CheckButton trebleboostBtn_;
    Gtk::CheckButton stereoenhanceBtn_; 
    Gtk::CheckButton rhythmeBtn_;
    Gtk::CheckButton roomBtn_;
    Gtk::CheckButton roomBtn2_;
    Gtk::CheckButton harmonyBtn_;
    Gtk::CheckButton distortionBtn_;

    Gtk::HScale stereoenhanceScale_;
    Gtk::HScale roomScale_;
    Gtk::HScale room2Scale_;
    Gtk::HScale harmonyScale_;
    Gtk::HScale rhythmeScale_;
    Gtk::HScale distortionScale_;

    Gtk::Label stableLabel_;
    Gtk::Label developLabel_; 

    Gtk::Tooltips bassBoostTooltips_;
    Gtk::Tooltips bassBoost2Tooltips_;
    Gtk::Tooltips trebleBoostTooltips_;
    Gtk::Tooltips stereoenhanceTooltips_;
    Gtk::Tooltips roomTooltips_;
    Gtk::Tooltips room2Tooltips_;

    SaguaPreference& pref_;

    bool effectChanged_;

    void OnBtnStereoEnhance();
    void OnBtnRoom();
    void OnBtnRoom2();
    void OnBtnHarmony();
    void OnBtnRhythme();
    void OnBtnDistortion();
};

/// @brief GraphicEQ tab of Preference window.
class PrefWndGraphicEQTab
{
public:
    PrefWndGraphicEQTab(SaguaPreference& pref);
    ~PrefWndGraphicEQTab()
    {}

    void AppendTo(Gtk::Notebook& notebook, const char* label)
    {
        notebook.append_page(EQTable_, label);
    }

    bool EQChanged() const
    {
        return eqChanged_;
    }

    bool CheckEQChange();

    void Update();

private:
    PrefWndGraphicEQTab(const PrefWndGraphicEQTab& prefWndGraphicEQTab);
    PrefWndGraphicEQTab& operator=(const PrefWndGraphicEQTab& prefWndGraphicEQTab);

    Gtk::Table EQTable_;

    Gtk::HBox EQBox50_, EQBox100_, EQBox200_, EQBox400_, EQBox800_, EQBox1500_;
    Gtk::HBox EQBox3k_, EQBox5k_, EQBox7k_, EQBox10k_, EQBox15k_;

    Gtk::CheckButton activateBtn_;

    Gtk::HScale EQ50_, EQ100_, EQ200_, EQ400_, EQ800_, EQ1500_;
    Gtk::HScale EQ3k_, EQ5k_, EQ7k_, EQ10k_, EQ15k_;

    Gtk::Label titleLabel_;

    SaguaPreference& pref_;

    bool eqChanged_;

    void OnBtnGraphicEQ();
};

/// @brief Implementaion of PrefWnd.
class PrefWnd::PrefWndImpl
{
public:
    PrefWndImpl(Gtk::Window& parent, MainManager& mainManager);
    ~PrefWndImpl()
    {}

    Gtk::Window window_;
    Gtk::VBox mainBox_;
    Gtk::Notebook mainNotebook_;
    Gtk::HButtonBox mainBtnBox_;

    Gtk::Button cancelBtn_, okBtn_;

    MainManager& mainManager_;

    PrefWndGeneralTab prefWndGeneralTab_;
    PrefWndAudioDriverTab prefWndAudioDriverTab_;
    PrefWndEffectorTab prefWndEffectorTab_;
    PrefWndGraphicEQTab prefWndGraphicEQTab_;

    void OnBtnCancel();
    void OnBtnOk();
};

PrefWnd::PrefWnd(Gtk::Window& parent, MainManager& mainManager)
    : impl_(new PrefWndImpl(parent, mainManager))
{}

PrefWnd::~PrefWnd()
{}

void PrefWnd::Run()
{
    Gtk::Main::run(impl_->window_);
}

PrefWndGeneralTab::PrefWndGeneralTab(SaguaPreference& pref)
    : compactBgPixmap_(gettext("Background image file used when main window is compact mode."), gettext("Choose background image file")),
      titleLabel_(gettext("<span weight=\"bold\">General</span>"), 0.0, 0.5),
      trayIconActionLabel_(gettext("Tray icon click action : "), 0.0, 0.5),
      assumeCharsetLabel_(gettext("Assume charset : "), 0.0, 0.5),
      compactBgPixmapLabel_(gettext("Compact mode background : "), 0.0, 0.5),
      mixerDevLabel_(gettext("Mixer device : "), 0.0, 0.5),
#ifdef ENABLE_HTTP
      convertUrlBtn_(gettext("Convert URL to current encoding")),
#endif
      pref_(pref),
      mixerChanged_(false),
      compactBgChanged_(false),
      assumeCharsetChanged_(false)
{
    generalTable_.resize(3, 19);

    generalTable_.set_border_width(12);
    generalTable_.set_row_spacings(6);
    generalTable_.set_col_spacing(0, 12);
    generalTable_.set_col_spacing(1, 12);

    titleLabel_.set_use_markup(true);
    generalTable_.attach(titleLabel_, 0, 3, 0, 1, Gtk::FILL);

    generalTable_.attach(trayIconActionLabel_, 1, 2, 1, 2, Gtk::FILL);

    Gtk::Menu_Helpers::MenuList& list = trayIconActionMenu_.items();
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Open window"), sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "openwin") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("File info."), sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "fileinfo") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Stop"), sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "stop") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Pause"), sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "pause") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Play"), sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "play") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Prev"), sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "prev") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Next"), sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "next") ) );

    const std::string& currentTrayIconAction(pref_.GetString("GeneralTrayIconAction"));

    if(currentTrayIconAction == "openwin")
        trayIconActionMenu_.set_active(0);

    else if(currentTrayIconAction == "fileinfo")
        trayIconActionMenu_.set_active(1);

    else if(currentTrayIconAction == "stop")
        trayIconActionMenu_.set_active(2);

    else if(currentTrayIconAction == "pause")
        trayIconActionMenu_.set_active(3);

    else if(currentTrayIconAction == "play")
        trayIconActionMenu_.set_active(4);

    else if(currentTrayIconAction == "prev")
        trayIconActionMenu_.set_active(5);

    else if(currentTrayIconAction == "next")
        trayIconActionMenu_.set_active(6);

    else
        trayIconActionMenu_.set_active(0);

    trayIconActionOptionMenu_.set_menu(trayIconActionMenu_);
    trayIconActionTooltips_.set_tip(trayIconActionOptionMenu_, gettext("Action when you click a tray icon in first button"));
    generalTable_.attach(trayIconActionOptionMenu_, 2, 3, 1, 2, Gtk::FILL);

    if(Glib::get_charset())
        assumeCharset_.set_text(pref_.GetString("GeneralAssumeCharset"));
    else
        assumeCharset_.set_sensitive(false);

    assumeCharsetTooltips_.set_tip(assumeCharset_,
                                   gettext("Assume charset when convert not valid UTF-8 string to UTF-8 and machine's charset is UTF-8"));	
    generalTable_.attach(assumeCharsetLabel_, 1, 2, 2, 3, Gtk::FILL);
    generalTable_.attach(assumeCharset_, 2, 3, 2, 3, Gtk::FILL); 

    generalTable_.attach(compactBgPixmapLabel_, 1, 2, 3, 4, Gtk::FILL);

    compactBgPixmap_.SetEntryText(pref_.GetString("GeneralCompactBgPixmap"));

    generalTable_.attach(compactBgPixmap_, 2, 3, 3, 4, Gtk::FILL);

    generalTable_.attach(mixerDevLabel_, 1, 2, 4, 5, Gtk::FILL);

    mixerDev_.set_text(pref_.GetString("MixerDev"));

    generalTable_.attach(mixerDev_, 2, 3, 4, 5, Gtk::FILL);

#ifdef ENABLE_HTTP
    convertUrlBtn_.set_active(pref_.GetBool("GeneralConvertUrlToCurrentEncoding"));

    if(Glib::get_charset())
        convertUrlBtn_.set_sensitive(false);

    generalTable_.attach(convertUrlBtn_, 1, 3, 5, 6, Gtk::FILL);
    generalTable_.attach(*manage(new Gtk::Label), 0, 3, 6, 19, Gtk::FILL);
#else
    generalTable_.attach(*manage(new Gtk::Label), 0, 3, 5, 19, Gtk::FILL);
#endif
}

void PrefWndGeneralTab::Update()
{
    if(mixerDev_.get_text() != pref_.GetString("MixerDev")) {

        mixerChanged_ = true;
        pref_.Set("MixerDev", mixerDev_.get_text());
    }

    Glib::ustring bgFile(Glib::filename_from_utf8(compactBgPixmap_.GetEntryText()));
    if(bgFile != pref_.GetString("GeneralCompactBgPixmap")) {

        compactBgChanged_ = true;
        pref_.Set("GeneralCompactBgPixmap", bgFile);
    }

    if(assumeCharset_.get_text() != pref_.GetString("GeneralAssumeCharset")) {

        assumeCharsetChanged_ = true;
        pref_.Set("GeneralAssumeCharset", assumeCharset_.get_text());
    }

#ifdef ENABLE_HTTP
    pref_.Set("GeneralConvertUrlToCurrentEncoding", convertUrlBtn_.get_active());
#endif
}

/// @brief Callback function used when trayicon action option menu is clicked.
void PrefWndGeneralTab::OnMenuTrayIconAction(const char* action)
{
    pref_.Set("GeneralTrayIconAction", action);
}

/// @todo Remove duplicated code on audio driver and audio driver option.
PrefWndAudioDriverTab::PrefWndAudioDriverTab(MainManager& mainManager)
    : warningIcon_(Gtk::Stock::DIALOG_WARNING, Gtk::ICON_SIZE_LARGE_TOOLBAR),
      mainTable_(5, 19),
      defaultBtn_(driverBtnGroup_, gettext("Use default driver\n(auto detect)")),
      alsa5Btn_(driverBtnGroup_, "Alsa 0.5.x"),
      alsa9Btn_(driverBtnGroup_, "Alsa 0.9.x"),
      esdBtn_(driverBtnGroup_, "ESD"),
      irixBtn_(driverBtnGroup_, "Irix"),
      ossBtn_(driverBtnGroup_, "OSS"),
      sunBtn_(driverBtnGroup_, "SUN"),
      aixsBtn_(driverBtnGroup_, "Aixs"),
      testBtn_(gettext("Test")),
      titleLabel_(gettext("<span weight=\"bold\">Audio Driver</span>"), 0.0, 0.5),
      alsa5CardLabel_("Card : ", 0.0, 0.5),
      alsa5DevLabel_("Device : ", 0.0, 0.5),
      alsa9DevLabel_("Device : ", 0.0, 0.5),
      ossDspLabel_(gettext("Path : "), 0.0, 0.5),
      sunDevLabel_(gettext("Path : "), 0.0, 0.5),
      aixsDevLabel_(gettext("Path : "), 0.0, 0.5),
      testLabel_(gettext("<span weight=\"bold\">Audio Driver Test</span>"), 0.0, 0.5),
      warningLabel_(gettext("Change will stop your playing!")),
      selectedDriver_(mainManager.GetPreference().GetString("AudioDriver")),
      mainManager_(mainManager),
      audioChanged_(false)
{
    mainTable_.set_border_width(12);
    mainTable_.set_row_spacings(6);
    mainTable_.set_col_spacing(0, 12);
    mainTable_.set_col_spacing(1, 12);

    titleLabel_.set_use_markup(true);
    mainTable_.attach(titleLabel_, 0, 5, 0, 1, Gtk::FILL);

    warningBox_.pack_start(warningIcon_, Gtk::PACK_SHRINK);
    warningBox_.pack_start(warningLabel_, Gtk::PACK_SHRINK);
    mainTable_.attach(warningBox_, 1, 5, 1, 2, Gtk::FILL);

    defaultBtn_.signal_clicked().connect( sigc::bind(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "default") );
    mainTable_.attach(defaultBtn_, 1, 4, 2, 3, Gtk::FILL);
    mainTable_.attach(*manage(new Gtk::Label), 4, 5, 2, 3, Gtk::EXPAND);

    alsa5Btn_.signal_clicked().connect( sigc::bind(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "alsa") );
    alsa5Tooltips_.set_tip(alsa5Btn_, gettext("Driver for alsa 0.5.x"));
    mainTable_.attach(alsa5Btn_, 1, 4, 3, 4, Gtk::FILL);

    mainTable_.attach(alsa5CardLabel_, 2, 3, 4, 5, Gtk::FILL);
    alsa5CardOptionBox_.set_spacing(12);
    alsa5CardOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);

    const SaguaPreference& pref(mainManager_.GetPreference());

    alsa5Card_.set_text(pref.GetString("AudioAlsa5Card"));
    alsa5CardOptionBox_.pack_start(alsa5Card_);
    mainTable_.attach(alsa5CardOptionBox_, 3, 5, 4, 5, Gtk::FILL);

    mainTable_.attach(alsa5DevLabel_, 2, 3, 5, 6, Gtk::FILL);
    alsa5DevOptionBox_.set_spacing(12);
    alsa5DevOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);

    alsa5Dev_.set_text(pref.GetString("AudioAlsa5Dev"));
    alsa5DevOptionBox_.pack_start(alsa5Dev_);
    mainTable_.attach(alsa5DevOptionBox_, 3, 5, 5, 6, Gtk::FILL);

    alsa9Btn_.signal_clicked().connect( sigc::bind(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "alsa09") );
    alsa9Tooltips_.set_tip(alsa9Btn_, gettext("Driver for alsa 0.9.x and 1.0.x"));
    mainTable_.attach(alsa9Btn_, 1, 4, 6, 7, Gtk::FILL);

    mainTable_.attach(alsa9DevLabel_, 2, 3, 7, 8, Gtk::FILL);
    alsa9OptionBox_.set_spacing(12);
    alsa9OptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);

    alsa9Dev_.set_text(pref.GetString("AudioAlsa9Dev"));
    alsa9DeviceTooltips_.set_tip(alsa9Dev_, gettext("ex) hw:0,0 or ens1371(naming in asoundrc)"));
    alsa9OptionBox_.pack_start(alsa9Dev_);
    mainTable_.attach(alsa9OptionBox_, 3, 5, 7, 8, Gtk::FILL);

    esdBtn_.signal_clicked().connect( sigc::bind(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "esd") );
    mainTable_.attach(esdBtn_, 1, 4, 8, 9, Gtk::FILL);

    irixBtn_.signal_clicked().connect( sigc::bind(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "irix") );
    mainTable_.attach(irixBtn_, 1, 4, 9, 10, Gtk::FILL);

    ossBtn_.signal_clicked().connect( sigc::bind(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "oss") );
    mainTable_.attach(ossBtn_, 1, 4, 10, 11, Gtk::FILL);

    mainTable_.attach(ossDspLabel_, 2, 3, 11, 12, Gtk::FILL);
    ossOptionBox_.set_spacing(12);
    ossOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);

    ossDsp_.set_text(pref.GetString("AudioOssDsp"));
    ossOptionBox_.pack_start(ossDsp_);
    mainTable_.attach(ossOptionBox_, 3, 5, 11, 12, Gtk::FILL);

    sunBtn_.signal_clicked().connect( sigc::bind(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "sun") );
    mainTable_.attach(sunBtn_, 1, 4, 12, 13, Gtk::FILL);

    mainTable_.attach(sunDevLabel_, 2, 3, 13, 14, Gtk::FILL);
    sunOptionBox_.set_spacing(12);
    sunOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);

    sunDev_.set_text(pref.GetString("AudioSunDev"));
    sunOptionBox_.pack_start(sunDev_);
    mainTable_.attach(sunOptionBox_, 3, 5, 13, 14, Gtk::FILL);

    aixsBtn_.signal_clicked().connect( sigc::bind(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "aixs") );
    mainTable_.attach(aixsBtn_, 1, 4, 14, 15, Gtk::FILL);

    mainTable_.attach(aixsDevLabel_, 2, 3, 15, 16, Gtk::FILL);
    aixsOptionBox_.set_spacing(12);
    aixsOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);

    aixsDev_.set_text(pref.GetString("AudioAixsDev"));
    aixsOptionBox_.pack_start(aixsDev_);
    mainTable_.attach(aixsOptionBox_, 3, 5, 15, 16, Gtk::FILL);

    testLabel_.set_use_markup(true);
    mainTable_.attach(testLabel_, 0, 5, 17, 18, Gtk::FILL);

    testBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriverTest) );
    mainTable_.attach(testBtn_, 1, 3, 18, 19, Gtk::FILL);
    mainTable_.attach(testResultLabel_, 3, 5, 18, 19, Gtk::FILL);

    alsa5CardOptionBox_.set_sensitive(false);
    alsa5DevOptionBox_.set_sensitive(false);
    alsa9OptionBox_.set_sensitive(false);
    ossOptionBox_.set_sensitive(false);
    sunOptionBox_.set_sensitive(false);
    aixsOptionBox_.set_sensitive(false);

    if(selectedDriver_ == "alsa") {

        alsa5Btn_.set_active(true);
        alsa5CardOptionBox_.set_sensitive(true);
        alsa5DevOptionBox_.set_sensitive(true);

    } else if(selectedDriver_ == "alsa09") {

        alsa9Btn_.set_active(true);
        alsa9OptionBox_.set_sensitive(true);

    } else if(selectedDriver_ == "esd") {

        esdBtn_.set_active(true);

    } else if(selectedDriver_ == "irix") {

        irixBtn_.set_active(true);

    } else if(selectedDriver_ == "oss") {

        ossBtn_.set_active(true);
        ossOptionBox_.set_sensitive(true);

    } else if(selectedDriver_ == "sun") {

        sunBtn_.set_active(true);
        sunOptionBox_.set_sensitive(true);

    } else if(selectedDriver_ == "aixs") {

        aixsBtn_.set_active(true);
        aixsOptionBox_.set_sensitive(true);

    } else {

        defaultBtn_.set_active(true);
    }

    audioDriverWindow_.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    audioDriverWindow_.add(mainTable_);
}

bool PrefWndAudioDriverTab::CheckDriverChange()
{
    const SaguaPreference& pref(mainManager_.GetPreference());

    if(selectedDriver_ != pref.GetString("AudioDriver"))

        return true;

    else {

        if(selectedDriver_ == "alsa") {

            if(alsa5Card_.get_text() != pref.GetString("AudioAlsa5Card") ||
               alsa5Dev_.get_text() != pref.GetString("AudioAlsa5Dev"))

                return true;
            else
                return false;

        } else if(selectedDriver_ == "alsa09") {

            if(alsa9Dev_.get_text() != pref.GetString("AudioAlsa9Dev"))

                return true;
            else
                return false;

        } else if(selectedDriver_ == "oss") {

            if(ossDsp_.get_text() != pref.GetString("AudioOssDsp"))

                return true;
            else
                return false;

        } else if(selectedDriver_ == "sun") {

            if(sunDev_.get_text() != pref.GetString("AudioSunDev"))

                return true;
            else
                return false;

        } else if(selectedDriver_ == "aixs") {

            if(aixsDev_.get_text() != pref.GetString("AudioAixsDev"))

                return true;
            else
                return false;

        }
    }

    return false;
}

void PrefWndAudioDriverTab::Update()
{
    audioChanged_ = CheckDriverChange();

    SaguaPreference& pref(mainManager_.GetPreference());

    pref.Set("AudioDriver", selectedDriver_);
    pref.Set("AudioAlsa5Card", alsa5Card_.get_text());
    pref.Set("AudioAlsa5Dev", alsa5Dev_.get_text());
    pref.Set("AudioAlsa9Dev", alsa9Dev_.get_text());
    pref.Set("AudioOssDsp", ossDsp_.get_text());
    pref.Set("AudioSunDev", sunDev_.get_text());
    pref.Set("AudioAixsDev", aixsDev_.get_text());
}

/// @brief Callback function used when any audio driver check button is clicked.
void PrefWndAudioDriverTab::OnBtnAudioDriver(const char* driver)
{
    selectedDriver_ = driver;

    alsa5CardOptionBox_.set_sensitive(false);
    alsa5DevOptionBox_.set_sensitive(false);
    alsa9OptionBox_.set_sensitive(false);
    ossOptionBox_.set_sensitive(false);
    sunOptionBox_.set_sensitive(false);
    aixsOptionBox_.set_sensitive(false);

    if(selectedDriver_ == "alsa") {

        alsa5CardOptionBox_.set_sensitive(true);
        alsa5DevOptionBox_.set_sensitive(true);

    } else if(selectedDriver_ == "alsa09") {

        alsa9OptionBox_.set_sensitive(true);

    } else if(selectedDriver_ == "oss") {

        ossOptionBox_.set_sensitive(true);

    } else if(selectedDriver_ == "sun") {

        sunOptionBox_.set_sensitive(true);

    } else if(selectedDriver_ == "aixs") {

        aixsOptionBox_.set_sensitive(true);

    }
}

/// @brief Callback function used when audio driver test button is clicked.
/// @sa MainManager::AudioTestDriver
void PrefWndAudioDriverTab::OnBtnAudioDriverTest()
{
    std::map<std::string, std::string> option;

    if(selectedDriver_ == "alsa" ) {

        option["card"] = alsa5Card_.get_text();
        option["dev"] = alsa5Dev_.get_text();

    } else if(selectedDriver_ == "alsa09") {

        option["dev"] = alsa9Dev_.get_text();

    } else if(selectedDriver_ == "oss") {

        option["dsp"] = ossDsp_.get_text();

    } else if(selectedDriver_ == "sun") {

        option["dev"] = sunDev_.get_text();

    } else if(selectedDriver_ == "aixs") {

        option["dev"] = aixsDev_.get_text();
    }

    if(mainManager_.TestDriver(selectedDriver_.c_str(), option))
        testResultLabel_.set_text(gettext("Success"));
    else
        testResultLabel_.set_text(gettext("Fail"));	
}

PrefWndEffectorTab::PrefWndEffectorTab(SaguaPreference& pref)
    : bassboostBtn_("Bass Boost"),
      bassboost2Btn_("Bass Boost2"),
      trebleboostBtn_("High Frequency Boost"),
      stereoenhanceBtn_("Stereo Enhance"),
      rhythmeBtn_("Rhythme Effect"),
      roomBtn_("Room Effect"),
      roomBtn2_("Room2 Effect"),
      harmonyBtn_("Harmony Effect"),
      distortionBtn_("Distortion Effect"),
      stereoenhanceScale_(0.1, 2.1, 0.1),
      roomScale_(0.1, 1.1, 0.1),
      room2Scale_(0.1, 1.1, 0.1),
      harmonyScale_(0.1, 1.1, 0.1),
      rhythmeScale_(0.1, 1.6, 0.1),
      distortionScale_(1.0, 10.1, 0.1),
      stableLabel_(gettext("<span weight=\"bold\">Effector</span>"), 0.0, 0.5),
      developLabel_(gettext("<span weight=\"bold\">Experimental Effector</span>"), 0.0, 0.5),
      pref_(pref),
      effectChanged_(false)
{
    effectorTable_.resize(3, 19);

    effectorTable_.set_border_width(12);
    effectorTable_.set_row_spacings(6);
    effectorTable_.set_col_spacing(0, 12);
    effectorTable_.set_col_spacing(1, 12);

    stableLabel_.set_use_markup(true);
    effectorTable_.attach(stableLabel_, 0, 3, 0, 1, Gtk::FILL);

    bassboostBtn_.set_active(pref_.GetBool("EfxBassBoost"));
    bassBoostTooltips_.set_tip(bassboostBtn_, gettext("Boost low frequency band"));
    effectorTable_.attach(bassboostBtn_, 1, 2, 1, 2, Gtk::FILL);
    effectorTable_.attach(*manage(new Gtk::Label), 2, 3, 1, 2, Gtk::EXPAND);

    bassboost2Btn_.set_active(pref_.GetBool("EfxBassBoost2"));
    bassBoost2Tooltips_.set_tip(bassboost2Btn_, gettext("More low frequency band boost"));
    effectorTable_.attach(bassboost2Btn_, 1, 2, 2, 3, Gtk::FILL);

    trebleboostBtn_.set_active(pref_.GetBool("EfxTrebleBoost"));
    trebleBoostTooltips_.set_tip(trebleboostBtn_, gettext("Boost high frequency band"));
    effectorTable_.attach(trebleboostBtn_, 1, 2, 3, 4, Gtk::FILL);

    stereoenhanceBtn_.set_active(pref_.GetBool("EfxStereoEnhance"));
    stereoenhanceBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnStereoEnhance) );
    stereoenhanceTooltips_.set_tip(stereoenhanceBtn_, gettext("Stereo enhance effect"));
    effectorTable_.attach(stereoenhanceBtn_, 1, 2, 4, 5, Gtk::FILL);
    stereoenhanceScale_.set_value_pos(Gtk::POS_RIGHT);
    stereoenhanceScale_.set_draw_value();
    stereoenhanceScale_.set_value(pref_.GetDouble("EfxStereoEnhanceAmount"));
    stereoenhanceScale_.set_sensitive(pref_.GetBool("EfxStereoEnhance"));
    stereoenhanceBox_.set_spacing(12);
    stereoenhanceBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    stereoenhanceBox_.pack_start(stereoenhanceScale_);
    effectorTable_.attach(stereoenhanceBox_, 1, 3, 5, 6, Gtk::FILL);
    effectorTable_.set_row_spacing(2, 3);

    effectorTable_.attach(*manage(new Gtk::Label), 0, 3, 6, 7, Gtk::FILL);

    developLabel_.set_use_markup(true);
    effectorTable_.attach(developLabel_, 0, 3, 7, 8, Gtk::FILL);

    roomBtn_.set_active(pref_.GetBool("EfxRoom"));
    roomBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnRoom) );
    roomTooltips_.set_tip(roomBtn_, gettext("Experimental space effector"));
    effectorTable_.attach(roomBtn_, 1, 2, 8, 9, Gtk::FILL);
    roomScale_.set_value_pos(Gtk::POS_RIGHT);
    roomScale_.set_draw_value();
    roomScale_.set_value(pref_.GetDouble("EfxRoomAmount"));
    roomScale_.set_sensitive(pref_.GetBool("EfxRoom"));
    roomBox_.set_spacing(12);
    roomBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    roomBox_.pack_start(roomScale_);
    effectorTable_.attach(roomBox_, 1, 3, 9, 10, Gtk::FILL);
    effectorTable_.set_row_spacing(8, 3);

    roomBtn2_.set_active(pref_.GetBool("EfxRoom2"));
    roomBtn2_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnRoom2) );
    room2Tooltips_.set_tip(roomBtn2_, gettext("Another experimental space effector"));
    effectorTable_.attach(roomBtn2_, 1, 2, 10, 11, Gtk::FILL);
    room2Scale_.set_value_pos(Gtk::POS_RIGHT);
    room2Scale_.set_draw_value();
    room2Scale_.set_value(pref_.GetDouble("EfxRoom2Amount"));
    room2Scale_.set_sensitive(pref_.GetBool("EfxRoom2"));
    room2Box_.set_spacing(12);
    room2Box_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    room2Box_.pack_start(room2Scale_);
    effectorTable_.attach(room2Box_, 1, 3, 11, 12, Gtk::FILL);
    effectorTable_.set_row_spacing(10, 3);

    harmonyBtn_.set_active(pref_.GetBool("EfxHarmony"));
    harmonyBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnHarmony) );
    effectorTable_.attach(harmonyBtn_, 1, 2, 12, 13, Gtk::FILL);
    harmonyScale_.set_value_pos(Gtk::POS_RIGHT);
    harmonyScale_.set_draw_value();
    harmonyScale_.set_value(pref_.GetDouble("EfxHarmonyAmount"));
    harmonyScale_.set_sensitive(pref_.GetBool("EfxHarmony"));
    harmonyBox_.set_spacing(12);
    harmonyBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    harmonyBox_.pack_start(harmonyScale_);
    effectorTable_.attach(harmonyBox_, 1, 3, 13, 14, Gtk::FILL);
    effectorTable_.set_row_spacing(12, 3);

    rhythmeBtn_.set_active(pref_.GetBool("EfxRhythme"));
    rhythmeBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnRhythme) );
    effectorTable_.attach(rhythmeBtn_, 1, 2, 14, 15, Gtk::FILL);
    rhythmeScale_.set_value_pos(Gtk::POS_RIGHT);
    rhythmeScale_.set_draw_value();
    rhythmeScale_.set_value(pref_.GetDouble("EfxRhythmeAmount"));
    rhythmeScale_.set_sensitive(pref_.GetBool("EfxRhythme"));
    rhythmeBox_.set_spacing(12);
    rhythmeBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    rhythmeBox_.pack_start(rhythmeScale_);
    effectorTable_.attach(rhythmeBox_, 1, 3, 15, 16, Gtk::FILL);
    effectorTable_.set_row_spacing(14, 3);

    distortionBtn_.set_active(pref_.GetBool("EfxDistortion"));
    distortionBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnDistortion) );
    effectorTable_.attach(distortionBtn_, 1, 2, 16, 17, Gtk::FILL);
    distortionScale_.set_value_pos(Gtk::POS_RIGHT);
    distortionScale_.set_draw_value();
    distortionScale_.set_value(pref_.GetDouble("EfxDistortionAmount"));
    distortionScale_.set_sensitive(pref_.GetBool("EfxDistortion"));
    distortionBox_.set_spacing(12);
    distortionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    distortionBox_.pack_start(distortionScale_);
    effectorTable_.attach(distortionBox_, 1, 3, 17, 18, Gtk::FILL);
    effectorTable_.set_row_spacing(16, 3);

    effectorTable_.attach(*manage(new Gtk::Label), 0, 3, 18, 19, Gtk::FILL);
}

bool PrefWndEffectorTab::CheckEffectChange()
{
    return stereoenhanceScale_.get_value() != pref_.GetDouble("EfxStereoEnhanceAmount") ||
        roomScale_.get_value() != pref_.GetDouble("EfxRoomAmount") ||
        room2Scale_.get_value() != pref_.GetDouble("EfxRoom2Amount") ||
        harmonyScale_.get_value() != pref_.GetDouble("EfxHarmonyAmount") ||
        rhythmeScale_.get_value() != pref_.GetDouble("EfxRhythmeAmount") ||
        distortionScale_.get_value() != pref_.GetDouble("EfxDistortionAmount") ||
        bassboostBtn_.get_active() != pref_.GetBool("EfxBassBoost") ||
        bassboost2Btn_.get_active() != pref_.GetBool("EfxBassBoost2") ||
        stereoenhanceBtn_.get_active() != pref_.GetBool("EfxStereoEnhance") ||
        trebleboostBtn_.get_active() != pref_.GetBool("EfxTrebleBoost") ||
        roomBtn_.get_active() != pref_.GetBool("EfxRoom") ||
        roomBtn2_.get_active() != pref_.GetBool("EfxRoom2") ||
        harmonyBtn_.get_active() != pref_.GetBool("EfxHarmony") ||
        rhythmeBtn_.get_active() != pref_.GetBool("EfxRhythme") ||
        distortionBtn_.get_active() != pref_.GetBool("EfxDistortion");
}

void PrefWndEffectorTab::Update()
{
    effectChanged_ = CheckEffectChange();

    pref_.Set("EfxStereoEnhanceAmount", stereoenhanceScale_.get_value());
    pref_.Set("EfxRoomAmount", roomScale_.get_value());
    pref_.Set("EfxRoom2Amount", room2Scale_.get_value());
    pref_.Set("EfxHarmonyAmount", harmonyScale_.get_value());
    pref_.Set("EfxRhythmeAmount", rhythmeScale_.get_value());
    pref_.Set("EfxDistortionAmount", distortionScale_.get_value());

    pref_.Set("EfxBassBoost", bassboostBtn_.get_active());
    pref_.Set("EfxBassBoost2", bassboost2Btn_.get_active());
    pref_.Set("EfxStereoEnhance", stereoenhanceBtn_.get_active());
    pref_.Set("EfxTrebleBoost", trebleboostBtn_.get_active());
    pref_.Set("EfxRoom", roomBtn_.get_active());
    pref_.Set("EfxRoom2", roomBtn2_.get_active());
    pref_.Set("EfxHarmony", harmonyBtn_.get_active());
    pref_.Set("EfxRhythme", rhythmeBtn_.get_active());
    pref_.Set("EfxDistortion", distortionBtn_.get_active());

    pref_.UpdateEffectUse();
}

/// @brief Callback function used when stereo enhance effector check button is clicked.
void PrefWndEffectorTab::OnBtnStereoEnhance()
{
    stereoenhanceScale_.set_sensitive(stereoenhanceBtn_.get_active());
}

/// @brief Callback function used when room effector check button is clicked.
void PrefWndEffectorTab::OnBtnRoom()
{
    roomScale_.set_sensitive(roomBtn_.get_active());
}

/// @brief Callback function used when room2 effector check button is clicked.
void PrefWndEffectorTab::OnBtnRoom2()
{
    room2Scale_.set_sensitive(roomBtn2_.get_active());
}

/// @brief Callback function used when harmony effector check button is clicked.
void PrefWndEffectorTab::OnBtnHarmony()
{
    harmonyScale_.set_sensitive(harmonyBtn_.get_active());
}

/// @brief Callback function used when rhythme effector check button is clicked.
void PrefWndEffectorTab::OnBtnRhythme()
{
    rhythmeScale_.set_sensitive(rhythmeBtn_.get_active());
}

/// @brief Callback function used when distortion effector check button is clicked.
void PrefWndEffectorTab::OnBtnDistortion()
{
    distortionScale_.set_sensitive(distortionBtn_.get_active());
}

PrefWndGraphicEQTab::PrefWndGraphicEQTab(SaguaPreference& pref)
    : activateBtn_( gettext("Use equalizer") ),
      // Is it efficient more than 6 dB? anyway...
      EQ50_(-12.0, 12.1, 0.1),
      EQ100_(-12.0, 12.1, 0.1),
      EQ200_(-12.0, 12.1, 0.1),
      EQ400_(-12.0, 12.1, 0.1),
      EQ800_(-12.0, 12.1, 0.1),
      EQ1500_(-12.0, 12.1, 0.1),
      EQ3k_(-12.0, 12.1, 0.1),
      EQ5k_(-12.0, 12.1, 0.1),
      EQ7k_(-12.0, 12.1, 0.1),
      EQ10k_(-12.0, 12.1, 0.1),
      EQ15k_(-12.0, 12.1, 0.1),
      titleLabel_(gettext("<span weight=\"bold\">Equalizer</span>"), 0.0, 0.5),
      pref_(pref),
      eqChanged_(false)
{
    EQTable_.resize(5, 19);

    EQTable_.set_border_width(12);
    EQTable_.set_row_spacings(6);
    EQTable_.set_col_spacing(0, 12);
    EQTable_.set_col_spacing(1, 12);

    titleLabel_.set_use_markup(true);
    EQTable_.attach(titleLabel_, 0, 5, 0, 1, Gtk::FILL);

    const bool activate(pref_.GetBool("EfxGraphiceq"));
    activateBtn_.set_active(activate);
    activateBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndGraphicEQTab::OnBtnGraphicEQ) );
    EQTable_.attach(activateBtn_, 1, 4, 1, 2, Gtk::FILL);
    EQTable_.attach(*manage(new Gtk::Label), 4, 5, 1, 2, Gtk::EXPAND);

    EQ50_.set_value_pos(Gtk::POS_RIGHT);
    EQ50_.set_draw_value();
    EQ50_.set_value(pref_.GetFloat("EfxGraphicEQGain50"));
    EQ50_.set_sensitive(activate);
    EQBox50_.set_spacing(12);
    EQBox50_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox50_.pack_start(EQ50_);
    EQTable_.attach(*manage(new Gtk::Label("50 Hz", 0.0, 0.5)), 2, 3, 2, 3, Gtk::FILL);
    EQTable_.attach(EQBox50_, 3, 5, 2, 3, Gtk::FILL);

    EQ100_.set_value_pos(Gtk::POS_RIGHT);
    EQ100_.set_draw_value();
    EQ100_.set_value(pref_.GetFloat("EfxGraphicEQGain100"));
    EQ100_.set_sensitive(activate);
    EQBox100_.set_spacing(12);
    EQBox100_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox100_.pack_start(EQ100_);
    EQTable_.attach(*manage(new Gtk::Label("100 Hz", 0.0, 0.5)), 2, 3, 3, 4, Gtk::FILL);
    EQTable_.attach(EQBox100_, 3, 5, 3, 4, Gtk::FILL);

    EQ200_.set_value_pos(Gtk::POS_RIGHT);
    EQ200_.set_draw_value();
    EQ200_.set_value(pref_.GetFloat("EfxGraphicEQGain200"));
    EQ200_.set_sensitive(activate);
    EQBox200_.set_spacing(12);
    EQBox200_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox200_.pack_start(EQ200_);
    EQTable_.attach(*manage(new Gtk::Label("200 Hz", 0.0, 0.5)), 2, 3, 4, 5, Gtk::FILL);
    EQTable_.attach(EQBox200_, 3, 5, 4, 5, Gtk::FILL);

    EQ400_.set_value_pos(Gtk::POS_RIGHT);
    EQ400_.set_draw_value();
    EQ400_.set_value(pref_.GetFloat("EfxGraphicEQGain400"));
    EQ400_.set_sensitive(activate);
    EQBox400_.set_spacing(12);
    EQBox400_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox400_.pack_start(EQ400_);
    EQTable_.attach(*manage(new Gtk::Label("400 Hz", 0.0, 0.5)), 2, 3, 5, 6, Gtk::FILL);
    EQTable_.attach(EQBox400_, 3, 5, 5, 6, Gtk::FILL);

    EQ800_.set_value_pos(Gtk::POS_RIGHT);
    EQ800_.set_draw_value();
    EQ800_.set_value(pref_.GetFloat("EfxGraphicEQGain800"));
    EQ800_.set_sensitive(activate);
    EQBox800_.set_spacing(12);
    EQBox800_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox800_.pack_start(EQ800_);
    EQTable_.attach(*manage(new Gtk::Label("800 Hz", 0.0, 0.5)), 2, 3, 6, 7, Gtk::FILL);
    EQTable_.attach(EQBox800_, 3, 5, 6, 7, Gtk::FILL);

    EQ1500_.set_value_pos(Gtk::POS_RIGHT);
    EQ1500_.set_draw_value();
    EQ1500_.set_value(pref_.GetFloat("EfxGraphicEQGain1.5k"));
    EQ1500_.set_sensitive(activate);
    EQBox1500_.set_spacing(12);
    EQBox1500_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox1500_.pack_start(EQ1500_);
    EQTable_.attach(*manage(new Gtk::Label("1.5 kHz", 0.0, 0.5)), 2, 3, 7, 8, Gtk::FILL);
    EQTable_.attach(EQBox1500_, 3, 5, 7, 8, Gtk::FILL);

    EQ3k_.set_value_pos(Gtk::POS_RIGHT);
    EQ3k_.set_draw_value();
    EQ3k_.set_value(pref_.GetFloat("EfxGraphicEQGain3k"));
    EQ3k_.set_sensitive(activate);
    EQBox3k_.set_spacing(12);
    EQBox3k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox3k_.pack_start(EQ3k_);
    EQTable_.attach(*manage(new Gtk::Label("3 kHz", 0.0, 0.5)), 2, 3, 8, 9, Gtk::FILL);
    EQTable_.attach(EQBox3k_, 3, 5, 8, 9, Gtk::FILL);

    EQ5k_.set_value_pos(Gtk::POS_RIGHT);
    EQ5k_.set_draw_value();
    EQ5k_.set_value(pref_.GetFloat("EfxGraphicEQGain5k"));
    EQ5k_.set_sensitive(activate);
    EQBox5k_.set_spacing(12);
    EQBox5k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox5k_.pack_start(EQ5k_);
    EQTable_.attach(*manage(new Gtk::Label("5 kHz", 0.0, 0.5)), 2, 3, 9, 10, Gtk::FILL);
    EQTable_.attach(EQBox5k_, 3, 5, 9, 10, Gtk::FILL);

    EQ7k_.set_value_pos(Gtk::POS_RIGHT);
    EQ7k_.set_draw_value();
    EQ7k_.set_value(pref_.GetFloat("EfxGraphicEQGain7k"));
    EQ7k_.set_sensitive(activate);
    EQBox7k_.set_spacing(12);
    EQBox7k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox7k_.pack_start(EQ7k_);
    EQTable_.attach(*manage(new Gtk::Label("7 kHz", 0.0, 0.5)), 2, 3, 10, 11, Gtk::FILL);
    EQTable_.attach(EQBox7k_, 3, 5, 10, 11, Gtk::FILL);

    EQ10k_.set_value_pos(Gtk::POS_RIGHT);
    EQ10k_.set_draw_value();
    EQ10k_.set_value(pref_.GetFloat("EfxGraphicEQGain10k"));
    EQ10k_.set_sensitive(activate);
    EQBox10k_.set_spacing(12);
    EQBox10k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox10k_.pack_start(EQ10k_);
    EQTable_.attach(*manage(new Gtk::Label("10 kHz", 0.0, 0.5)), 2, 3, 11, 12, Gtk::FILL);
    EQTable_.attach(EQBox10k_, 3, 5, 11, 12, Gtk::FILL);

    EQ15k_.set_value_pos(Gtk::POS_RIGHT);
    EQ15k_.set_draw_value();
    EQ15k_.set_value(pref_.GetFloat("EfxGraphicEQGain15k"));
    EQ15k_.set_sensitive(activate);
    EQBox15k_.set_spacing(12);
    EQBox15k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox15k_.pack_start(EQ15k_);
    EQTable_.attach(*manage(new Gtk::Label("15 kHz", 0.0, 0.5)), 2, 3, 12, 13, Gtk::FILL);
    EQTable_.attach(EQBox15k_, 3, 5, 12, 13, Gtk::FILL);

    EQTable_.attach(*manage(new Gtk::Label), 0, 5, 13, 19, Gtk::FILL);
}

bool PrefWndGraphicEQTab::CheckEQChange()
{
    return !activateBtn_.get_active() &&
        EQ50_.get_value() != pref_.GetFloat("EfxGraphicEQGain50") ||
        EQ100_.get_value() != pref_.GetFloat("EfxGraphicEQGain100") ||
        EQ200_.get_value() != pref_.GetFloat("EfxGraphicEQGain200") ||
        EQ400_.get_value() != pref_.GetFloat("EfxGraphicEQGain400") ||
        EQ800_.get_value() != pref_.GetFloat("EfxGraphicEQGain800") ||
        EQ1500_.get_value() != pref_.GetFloat("EfxGraphicEQGain1.5k") ||
        EQ3k_.get_value() != pref_.GetFloat("EfxGraphicEQGain3k") ||
        EQ5k_.get_value() != pref_.GetFloat("EfxGraphicEQGain5k") ||
        EQ7k_.get_value() != pref_.GetFloat("EfxGraphicEQGain7k") ||
        EQ10k_.get_value() != pref_.GetFloat("EfxGraphicEQGain10k") ||
        EQ15k_.get_value() != pref_.GetFloat("EfxGraphicEQGain15k");
}

void PrefWndGraphicEQTab::Update()
{
    eqChanged_ = CheckEQChange();

    pref_.Set("EfxGraphiceq", activateBtn_.get_active());

    pref_.Set("EfxGraphicEQGain50", static_cast<float>(EQ50_.get_value()));
    pref_.Set("EfxGraphicEQGain100", static_cast<float>(EQ100_.get_value()));
    pref_.Set("EfxGraphicEQGain200", static_cast<float>(EQ200_.get_value()));
    pref_.Set("EfxGraphicEQGain400", static_cast<float>(EQ400_.get_value()));
    pref_.Set("EfxGraphicEQGain800", static_cast<float>(EQ800_.get_value()));
    pref_.Set("EfxGraphicEQGain1.5k", static_cast<float>(EQ1500_.get_value()));
    pref_.Set("EfxGraphicEQGain3k", static_cast<float>(EQ3k_.get_value()));
    pref_.Set("EfxGraphicEQGain5k", static_cast<float>(EQ5k_.get_value()));
    pref_.Set("EfxGraphicEQGain7k", static_cast<float>(EQ7k_.get_value()));
    pref_.Set("EfxGraphicEQGain10k", static_cast<float>(EQ10k_.get_value()));
    pref_.Set("EfxGraphicEQGain15k", static_cast<float>(EQ15k_.get_value()));
}

/// @brief Callback function used when graphic EQ check button is clicked.
void PrefWndGraphicEQTab::OnBtnGraphicEQ()
{
    bool active(activateBtn_.get_active());

    EQ50_.set_sensitive(active);
    EQ100_.set_sensitive(active);
    EQ200_.set_sensitive(active);
    EQ400_.set_sensitive(active);
    EQ800_.set_sensitive(active);
    EQ1500_.set_sensitive(active);
    EQ3k_.set_sensitive(active);
    EQ5k_.set_sensitive(active);
    EQ7k_.set_sensitive(active);
    EQ10k_.set_sensitive(active);
    EQ15k_.set_sensitive(active);
}

PrefWnd::PrefWndImpl::PrefWndImpl(Gtk::Window& parent, MainManager& mainManager)
    : mainBtnBox_(Gtk::BUTTONBOX_END, 12),
      cancelBtn_(Gtk::Stock::CANCEL),
      okBtn_(Gtk::Stock::OK),
      mainManager_(mainManager),
      prefWndGeneralTab_(mainManager.GetPreference()),
      prefWndAudioDriverTab_(mainManager),
      prefWndEffectorTab_(mainManager.GetPreference()),
      prefWndGraphicEQTab_(mainManager.GetPreference())
{
    window_.set_transient_for(parent);

    window_.set_title(gettext("Preference"));
    window_.set_resizable(false);

    mainBox_.set_border_width(12);
    mainBox_.set_spacing(12);
    window_.add(mainBox_);

    prefWndGeneralTab_.AppendTo(mainNotebook_, gettext("General"));
    prefWndAudioDriverTab_.AppendTo(mainNotebook_, gettext("Audio Driver"));
    prefWndEffectorTab_.AppendTo(mainNotebook_, gettext("Effector"));
    prefWndGraphicEQTab_.AppendTo(mainNotebook_, gettext("Equalizer"));

    mainBox_.pack_start(mainNotebook_);

    cancelBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndImpl::OnBtnCancel) );
    mainBtnBox_.pack_start(cancelBtn_);
    okBtn_.signal_clicked().connect(sigc::mem_fun(*this, &PrefWndImpl::OnBtnOk) );
    mainBtnBox_.pack_start(okBtn_);

    mainBox_.pack_start(mainBtnBox_);

    window_.show_all_children();
}

/// @brief Callback function used when cancel button is clicked.
void PrefWnd::PrefWndImpl::OnBtnCancel()
{
    window_.hide();
}

/// @brief Callback function used when ok button is clicked.
///
/// Apply to current Preference and change audio driver if need.
void PrefWnd::PrefWndImpl::OnBtnOk()
{
    DEBUGTRACE("PrefWnd::PrefWndImpl::OnBtnOk");

    prefWndGeneralTab_.Update();

    prefWndAudioDriverTab_.Update();

    prefWndEffectorTab_.Update();

    prefWndGraphicEQTab_.Update();

    mainManager_.SetCurrentTrayIconAction();

    if(prefWndGeneralTab_.MixerDevChanged())
        mainManager_.Refresh();

    if(prefWndGeneralTab_.AssumeCharsetChanged())
        StringToUTF8::Instance().SetAssumeCharset(mainManager_.GetPreference().GetString("GeneralAssumeCharset"));

    if(mainManager_.GetPreference().GetBool("Effect")) {

        if(prefWndEffectorTab_.EffectChanged() ||
           prefWndGraphicEQTab_.EQChanged()) {


            mainManager_.GetPreference().EfxChangedSet();
        }
    }

    if(prefWndAudioDriverTab_.AudioDriverChanged()) {

        DEBUGTRACE_MSG("Audio driver changed, i'll try set new audio driver");

        mainManager_.SetChangedDriver();
    }

    if(prefWndGeneralTab_.CompactBgPixmapChanged())
        mainManager_.MainWndChangeCompactBgPixmap();

    mainManager_.SavePreference();

    window_.hide();
}
